@@ -54,6 +54,7 @@ TimeZone::Entry::Entry(
   parts.is_dst = is_dst;
 }
 
+
 TimeZone::Entry::Entry(
   int64_t const transition_time,
   TzFile::Type const& type)
@@ -221,13 +222,15 @@ TimeZone::get_parts_local(
 
 void
 TimeZone::extend_future(
-  int64_t const until)
+  int64_t until)
   const
 {
   if (future_.dst.abbreviation.empty())
     // No future DST.
     return;
 
+  until = std::min(until, EPOCH_TIME_MAX);
+
   if (until < stop_)
     // Already caught up.
     return;
@@ -246,33 +249,40 @@ TimeZone::extend_future(
   std::vector<Entry> entries;
   Datenum const datenum = stop_ / SECS_PER_DAY + DATENUM_UNIX_EPOCH;
   auto year = datenum_to_ordinal_date(datenum).year;
-  for (; true; ++year) {
+  for (; year < YEAR_END; ++year) {
     // Add the DST start transition.
-    auto d = weekday_of_month(
+    auto const ddst = weekday_of_month(
       year, start.month,
       start.week == 5 ? -1 : start.week,
       weekday::ENCODING_CLIB::decode(start.weekday));
-    auto t =
-      (d - DATENUM_UNIX_EPOCH) * SECS_PER_DAY
+    auto const tdst =
+      (int64_t) (ddst - DATENUM_UNIX_EPOCH) * SECS_PER_DAY
       + future_.start.ssm
       - future_.std.offset;
-    if (stop_ < t)
-      entries.emplace_back(t, future_.dst.offset, future_.dst.abbreviation, true);
 
     // Add the DST stop transition.
-    d = weekday_of_month(
+    auto const dstd = weekday_of_month(
       year, end.month,
       end.week == 5 ? -1 : end.week,
       weekday::ENCODING_CLIB::decode(end.weekday));
-    t =
-      (d - DATENUM_UNIX_EPOCH) * SECS_PER_DAY
+    auto const tstd =
+      (int64_t) (dstd - DATENUM_UNIX_EPOCH) * SECS_PER_DAY
       + future_.end.ssm
       - future_.dst.offset;
-    if (stop_ < t)
-      entries.emplace_back(t, future_.std.offset, future_.std.abbreviation, false);
 
-    if (stop < t)
+    if (stop_ <= tdst)
+      entries.emplace_back(tdst, future_.dst.offset, future_.dst.abbreviation, true);
+    if (stop_ <= tstd) {
+      entries.emplace_back(tstd, future_.std.offset, future_.std.abbreviation, false);
+      // Occasionally, STD start before DST start; in that case, swap them.
+      if (tstd < tdst && stop_ <= tdst)
+        std::swap(entries[entries.size() - 1], entries[entries.size() - 2]);
+    }
+
+    if (stop < tstd || stop < tdst) {
+      ++year;
       break;
+    }
   }
 
   // Append (to front) the new entries.
@@ -281,7 +291,6 @@ TimeZone::extend_future(
 
   // Note how far we got.
   stop_ = (jan1_datenum(year) - DATENUM_UNIX_EPOCH) * SECS_PER_DAY;
-  assert(until <= stop_);
 }
 
 
